%%%%% parameterization_Horvat_Tziperman.m %%%%%%%
% This file take the ouput data from netcdf_file_loc
% And evaluates a series of parameterizations of the eddy heat flux from
% the ocean to the ice. 
% Copyright C. Horvat, 2018. 

clear
close all

% Location of output netcdf files
netcdf_file_loc = 'output_files/';

% Various constants
rho_ice = 934; %kg/m^3
L_f = 334000; % j/kg
c_p = 4184; % j/kg/K
rho_0 = 1000; % kg/m^3
g = 9.81; % k/s^2
fcor = 1.4e-4; % 1/s
beta = 8e-4; % Haline Contraction


% Plotting tools 
% This is the extent of the domain we wish to plot
mer_lim = [0 30];
numz = 10; % Number of z levels to read
maxT = 40; % Number of time steps to plot

% Coefficients of fit
H = 5; % estimated depth of lens
C_nofit = .1; % Efficiency factor


%% Load in data

% Grid information
X = ncread([netcdf_file_loc 'state.nc'],'X');
Y = ncread([netcdf_file_loc 'state.nc'],'Y');

dz = 1; % Model output is levels, not z values. 
Z = -1*dz*ncread([netcdf_file_loc 'state.nc'],'diag_levels');

Y = Y(2:end-1); % Because of hard walls on top and bottom, only use 2-298. 
dx = X(2) - X(1);
dy = Y(2) - Y(1);

Time = ncread([netcdf_file_loc 'state.nc'],'T');
maxT = min(length(Time),maxT); 
Time = Time(1:maxT)/86400;

% Ice information
Ice = squeeze(ncread([netcdf_file_loc 'Surf.nc'],'SI_Fract',[1 2 1 1],[Inf 298 Inf maxT]));
Thick = squeeze(ncread([netcdf_file_loc 'Surf.nc'],'SI_Thick',[1 2 1 1],[Inf 298 Inf maxT]));
Vol = Ice.*Thick;

% Initial concentration
A_0 = squeeze(mean(mean(Ice(:,:,1,1),1),2));

% Dynamical information
V = ncread([netcdf_file_loc 'state.nc'],'VVEL',[1 2 1 1],[Inf 299 numz maxT]);
V = .5*(V(:,1:end-1,:,:) + V(:,2:end,:,:)); 
W = ncread([netcdf_file_loc 'state.nc'],'WVEL',[1 2 1 1],[Inf 298 numz maxT]);
Temp = ncread([netcdf_file_loc 'state.nc'],'THETA',[1 2 1 1],[Inf 298 numz maxT]);

% Density information
r0 = ncread([netcdf_file_loc 'state.nc'],'RHOAnoma',[1 2 1 1],[Inf 298 numz maxT]);
r0 = mean(mean(mean(r0(:,:,:))));
rho_0 = rho_0 + r0; % more accurate rep. of density anomaly. 
% Load in the density anomaly 
Rhoprime = -rho_0 + ncread([netcdf_file_loc 'state.nc'],'RHOAnoma',[1 2 1 1],[Inf 298 numz maxT]);
% Convert this to buoyancy
B = -g * (Rhoprime) / rho_0;

% Zonal mean fields
Bbar = (mean(B,1));
Vbar = (mean(V,1));
Wbar = (mean(W,1));
Tbar = mean(Temp,1);

clear r0 
%% Compute the heat flux that we can infer from this volume curve
% Remembering that dV/dt ~ Q/rho L_f
% We can compute Q by taking dV/dt. However this just gives the heat flux,
% integrated over the domain. What we really want is the heat flux, per square
% meter of ice. To get this we divide by the ice area.

A_ice = squeeze(mean(mean(Ice,1),2))/A_0;
V_ice = squeeze(mean(mean(Vol,1),2))/A_0;
Icevec = squeeze(mean(Ice(:,:,1),1));
notice = 1:149; % Initial open position
isice = 150:298; % Initial ice position

Q_melting = -gradient(V_ice,86400) * rho_ice * L_f ./ A_ice;
%% Now compute the parameterization

% Here we compute the parameterized heat flux.
% We obtain four "heat fluxes"

% 1) The actual heat flux used to melt ice. This is computed from the
% gradient in ice volume.
% 2a) A heat flux inferred from the eddy velocities \nabla dot (bar(u'b' + w'b'
% + v'b'))
% 3) A heat flux inferred from just the difference in ocean surface
% temperatures averaged over regions with or without ice, and over the top
% 5 meters.


Q_surface = ncread([netcdf_file_loc 'Surf.nc'],'surForcT',[1 2 1 1],[Inf 298 Inf maxT]);

FT_V = squeeze(mean(bsxfun(@minus,V,Vbar).*bsxfun(@minus,Temp,Tbar),1));
FT_W = squeeze(mean(bsxfun(@minus,W,Wbar).*bsxfun(@minus,Temp,Tbar),1));
[~,F_y,~] = gradient(squeeze(FT_V),Inf,Y,Inf);
[F_z,~,~] = gradient(squeeze(FT_W),Z,Inf,Inf);

ReyT = -F_y - F_z;
eddy_tend = ReyT(:,1:H,:);
eddy_flx = c_p*rho_0*abs(dz)*sum(eddy_tend(isice,:,:),2);
tot_eddy_HF = squeeze(mean(eddy_flx,1));


%% Temperature Diagnostics

% Compute the surface heat flux to open water
Q_in_open = -squeeze(mean(mean(Q_surface(:,notice,:),1),2)) .* A_0;
% The surface heat flux under ice
Q_under_ice = -squeeze(mean(mean(Q_surface(:,isice,:),1),2)) .* (1-A_0);
% Here is the total surface heat forcing. If the sea ice is melting at its
% surface this will not match the evolution of sea ice volume. 
Q_tot_surf = Q_in_open-Q_melting;

% Compute the average temperature in the left side of the domain
T_left = squeeze(mean(mean(mean(Temp(:,notice,1:H,:),1),2),3));

% Compute the average temperature if we accumulate the surface heat
% forcing. 
T_left_Qsurf = min(T_left(:)) + [0; cumsum((Q_tot_surf(1:end-1)/(H*c_p*rho_0))*86400)];

% Assume that the temperature under ice at freezing. 
T_right = -1.7578;
% Computed temperature field on either side of edge
dT_0 = (T_left - T_right);
dT_1 = T_left_Qsurf -T_right;
%% Velocity Diagnostics

[Bbar_Z,~,~] = gradient(squeeze(Bbar),dz,dy,1);
F_vpbp = squeeze(mean(bsxfun(@minus,V,Vbar).*(bsxfun(@minus,B,Bbar)),1));

Psi_AM = -(F_vpbp./Bbar_Z);
[V_AM,W_AM,~] = gradient(Psi_AM,dz,dy,1);
V_AM = -V_AM;

V_0 = -squeeze(mean(mean(V_AM(145:155,1:H,:),1),2));

% Estimate of V from fit
V_2 = .0025 + 0*V_0;

% Compute V from melting
Si(1) = 33;


for i = 2:maxT
    dS = (1/H) * 86400*(rho_ice / rho_0) * (Si(i-1)) * (-A_0 .* A_ice(i) .* Q_melting(i)) / (L_f * rho_ice);
    Si(i) = Si(i-1) + dS;
end

deltaB = -g * beta * (Si - Si(1));

% Estimated Length Scale
deltaX = 5000;

% Here is to find efficiency factor by fitting 
V_skeleton = H * abs((Si - Si(1))) * (g*beta) / (fcor*deltaX);
ft = fittype({'x'});
fitobj = fit(V_skeleton(~isnan(V_skeleton))',V_0(~isnan(V_skeleton)),ft);
C = fitobj.a; % Obtained from fit

% Use C = 0.1 for all output. 
V_1 = C_nofit*V_skeleton;


%% Now accumulate the eddy heat fluxes
eddy_HF_00 = c_p*rho_0 * V_0.* dT_0 / deltaX;
eddy_HF_10 = c_p*rho_0 * V_0.* dT_1 / deltaX;
eddy_HF_11 = c_p*rho_0 * V_1'.* dT_1 / deltaX;
eddy_HF_21 = c_p*rho_0 * V_2.* dT_1 / deltaX;

%%
close all
figure;

Ax{1} = subplot(221);
plot(Time,dT_0);
hold on
plot(Time,T_left_Qsurf + 1.7578);
legend('dT^0','dT^1','location','southeast')


Ax{2} =subplot(222);
plot(Time,V_0);
hold on
plot(Time,V_1);
plot(Time,V_2)
legend('V^0','V^1','V^2','location','northwest')

Ax{3} = subplot(2,1,2);
hold on
plot(Time,tot_eddy_HF);
plot(Time,eddy_HF_00);
plot(Time,eddy_HF_10);
plot(Time,eddy_HF_11);
plot(Time,eddy_HF_21);
legend('Q_{eddy}','Q^{00}','Q^{10}','Q^{11}','Q^{21}','location','northwest');

letter = {'(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)'};
titles = {'Temp','V','Q'};

for i = 1:length(Ax)
    
    
    axes(Ax{i})
    set(Ax{i},'ydir','normal','layer','top','fontname','helvetica','fontsize',10)
    grid(Ax{i},'on');
    title(Ax{i},titles{i},'interpreter','latex');
    box(Ax{i},'on');
    posy = get(Ax{i},'position');
    annotation('textbox',[posy(1)-.05 posy(2)+posy(4)+.05 .025 .025], ...
        'String',letter{i},'LineStyle','none','FontName','Helvetica', ...
        'FontSize',10,'Tag','legtag');
    
    
end


pos = [6.5 4]; 
set(gcf,'windowstyle','normal','position',[0 0 pos],'paperposition',[0 0 pos],'papersize',pos,'units','inches','paperunits','inches');
set(gcf,'windowstyle','normal','position',[0 0 pos],'paperposition',[0 0 pos],'papersize',pos,'units','inches','paperunits','inches');
saveas(gcf,[netcdf_file_loc 'Param.pdf']); 
saveas(gcf,[netcdf_file_loc 'Param.fig']); 

